;(function ($) {

    /**
     * Handles the logic for admin settings.
     *
     * @since 1.0.0
     * @class DiviMonk
     */
    window.DiviMonk = function( settings ) {
        if ( 'undefined' === typeof settings.container && 'undefined' === typeof settings.element ) {
            return;
        }

        if ( $(settings.container).length === 0 || $(settings.container).find(settings.element).length === 0 ) {
            return;
        }

        // Get variables from settings.
        this.settings   = settings;
        this.progress   = $(settings.progress);
        this.wrap       = $(settings.wrap);
        this.container  = this.wrap.find(settings.container);
        this.element    = this.container.find(settings.element);
        this.filter     = this.container.find(settings.filter);

        // Trigger init.
        this._init();
    };

    DiviMonk.prototype = {

        /**
         * Holds a flag for category filters.
         *
         * @since 1.0.0
         * @access private
		 * @var _filterActive
         */
        _filterActive: false,

        /**
         * Initialize methods.
         *
         * @since 1.0.0
         * @access private
		 * @method _bindEvents
         */
        _init: function()
        {
            this._bindEvents();
            this._updateURL();
        },

        /**
         * Bind events.
         *
         * @since 1.0.0
         * @access private
		 * @method _bindEvents
         */
        _bindEvents: function()
        {
            var self = this;

            // Fetch template.
            self.element.on('click', '.template-activate', function(e) {
                e.preventDefault();
                self._fetchTemplate( $(this) );
            });

            // Filter template.
            self.filter.on('click', 'a', function(e) {
                e.preventDefault();

                var selector = '.filter-' + $(this).data('filter');

                if ( 'undefined' === typeof selector || $(selector).length === 0 ) {
                    self._resetFilter();
                }
                else {
                    self._filterTemplate( $(selector) );
                    self.filter.find('a.active-filter').removeClass('active-filter');
                    $(this).addClass('active-filter');
                }
            });

            // Search template.
            self.container.on('keyup', '#wp-filter-search-input', function() {
                self._searchTemplate( $(this) );
            });

            // Effects.
            $(window).on('beforeunload', function() {
                self.container.fadeOut(2000);
                // Progress bar.
                var width = 1;
                var x = setInterval(function() {
                    if ( width >= 100 ) {
                        clearInterval(x);
                    } else {
                        width++;
                        self.progress.css('width', width + '%');
                    }
                }, 50);
            });
        },

        /**
         * Update window URL after reload trigger.
         *
         * @since 1.0.0
         * @access private
		 * @method _updateURL
         */
        _updateURL: function()
        {
            if ( history.pushState ) {
                if ( document.location.search.search( '&refresh' ) > -1 ) {
                    var url = document.location.href.split('&refresh')[0];
                    window.history.pushState( { path:url }, '', url );
                }
            }
        },

        /**
         * Filters template grid as per category.
         *
         * @since 1.0.0
         * @access private
		 * @method _filterTemplate
         * @param object $el
         */
        _filterTemplate: function($el)
        {
            this.element.hide();
            $el.fadeIn();
            this._filterActive = true;
        },

        /**
         * Reset applied filter.
         *
         * @since 1.0.0
         * @access private
		 * @method _resetFilter
         */
        _resetFilter: function()
        {
            this.element.fadeIn();
            this.filter.find('a.active-filter').removeClass('active-filter');
            this._filterActive = false;
        },

        /**
         * Search template as per category.
         *
         * @since 1.0.0
         * @access private
		 * @method _searchTemplate
         * @param object $search
         */
        _searchTemplate: function($search)
        {
            var self = this;

            if ( self._filterActive ) {
                self._resetFilter();
            }

            if ( $search.val().length >= 3 ) {

                var searchText = $search.val().toLowerCase().trim();

                self.element.hide();

                self.element.each(function() {
                    var templateTitle = $(this).find('.template-title').text().toLowerCase().trim();
                    if( templateTitle.search(searchText) !== -1 ) {
                        $(this).show();
                    } else {
                        $(this).hide();
                    }
                });
            }
            else {
                self._resetFilter();
            }

            return false;
        },

        /**
         * Show overlay to prevent user clicks while
         * downloading template.
         *
         * @since 1.0.0
         * @access private
		 * @method _showOverlay
         */
        _showOverlay: function()
        {
            var $overlay = this.wrap.find('.divi-monk-overlay');
            if ( $overlay.length > 0 ) {
                $overlay.fadeIn();
            }
        },

        /**
         * Hide overlay.
         *
         * @since 1.0.0
         * @access private
		 * @method _hideOverlay
         */
        _hideOverlay: function()
        {
            var $overlay = this.wrap.find('.divi-monk-overlay');
            if ( $overlay.length > 0 ) {
                $overlay.fadeOut();
            }
        },

        /**
         * Fetch template on download button click.
         *
         * @since 1.0.0
         * @access private
		 * @method _fetchTemplate
         * @param object $el
         */
        _fetchTemplate: function($el)
        {
            var self        = this;

            var id          = $el.data('template-id'),
                nonce       = $el.data('nonce'),
                downloading = $el.data('downloading'),
                downloaded  = $el.data('downloaded'),
                text        = $el.text(),
                parent      = $el.parents(self.settings.element);

            $el.addClass('disabled').text(downloading);

            self._showOverlay();
            self._log('Fetching template...', 'log');

            self._performAjax({
                'template_id'       : id,
                'template_nonce'    : nonce
            }, function( response ) {

                if ( response.success ) {
                    $el.text(downloaded);
                    parent.addClass('template-enabled');
                    parent.addClass('template-post-' + response.data.post_id);
                    self._log('Template has been imported successfully!', 'info');
                }
                else {
                    $el.text(text);
                    $el.removeClass('disabled');
                    self._log('Import Failed!', 'error');
                }

                self._hideOverlay();
                self._log(response, 'log');
            });
        },

        /**
         * Trigger ajax to fetch template.
         *
         * @since 1.0.0
         * @access private
		 * @method _performAjax
         * @param object data
         * @param function callback
         */
        _performAjax: function( data, callback )
        {
            var self = this;

            data = $.extend( {
                action      : 'dm_import_template',
                context     : 'et_builder_layouts',
                page        : 1,
            }, data );

            $.ajax({
                type    : 'post',
                url     : ajaxurl,
                data    : data,
                success : function( response ) {
                    if ( 'undefined' !== typeof response.page ) {

                        self._log('Paginating template images...', 'log');

                        self._performAjax( $.extend( data, {
							page                 : parseInt( response.page ) + 1,
							timestamp            : response.timestamp,
							file                 : null,
						} ), callback );

                        return;
                    }
                    self._log('Importing template...', 'log');
                    callback(response);
                }
            });
        },

        /**
         * Log message or errors in console.
         *
         * @since 1.0.0
         * @access private
		 * @method _log
         * @param mixed msg
         * @param string type
         */
        _log: function(msg, type)
        {
            if ( 'log' === type ) {
                console.log(msg);
            }
            if ( 'info' === type ) {
                console.info(msg);
            }
            if ( 'alert' === type ) {
                console.warn(msg);
            }
            if ( 'error' === type ) {
                console.error(msg);
            }
        }
    };

    $(document).ready(function() {
        // Initialize DiviMonk class.
        new DiviMonk({
            'progress'  : '.divi-monk-progress .divi-monk-progress-bar',
            'wrap'      : '.divi-monk-settings-wrap',
            'container' : '.divi-monk-templates',
            'element'   : '.divi-monk-template',
            'filter'    : '.divi-monk-template-categories'
        });
    });

})(jQuery);
