<?php

/**
 * Handles logic for the admin settings page.
 *
 * @since 1.0.0
 */
final class Divi_Monk_Admin_Settings {
    /**
	 * Holds any errors that may arise from
	 * saving admin settings.
	 *
	 * @since 1.0.0
	 * @var array $errors
	 */
	static public $errors = array();

    /**
     * Holds the templates count.
     *
     * @since 1.0.0
     * @var array
     */
	static public $templates_count = array();
	static public $templates_type = array();

    static public $settings_page = 'divi-monk-settings';

	/**
	 * Initializes the admin settings.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	static public function init()
	{
        self::flush_templates();
        self::flush_license();

        self::$templates_count = array(
            'page'  => 0,
            'row'   => 0,
        );

        self::$templates_type = array('layout', 'section', 'module');

		add_action( 'plugins_loaded', __CLASS__ . '::init_hooks' );
	}

	/**
	 * Adds the admin menu and enqueues CSS/JS if we are on
	 * the plugin's admin settings page.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	static public function init_hooks()
	{
		if ( ! is_admin() ) {
			return;
		}

        self::check_templates_data();

        add_action( 'admin_menu',           __CLASS__ . '::menu' );
        add_action( 'network_admin_menu',   __CLASS__ . '::menu' );

		if ( self::is_valid_page() ) {
            add_action( 'admin_enqueue_scripts', __CLASS__ . '::styles_scripts' );
			self::save();
		}
	}

    /**
	 * Enqueues the needed CSS/JS for the builder's admin settings page.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	static public function styles_scripts()
	{
		// Styles
		wp_enqueue_style( 'divi-monk-admin-style' );
		wp_enqueue_script( 'divi-monk-admin-script' );
	}

    /**
	 * Renders the admin settings menu.
	 *
	 * @since 1.1.5
	 * @return void
	 */
	static public function menu()
	{
		if ( current_user_can( 'manage_options' ) ) {

			$title = __('Divi Monk', 'divi-monk');
			$cap   = 'manage_options';
			$slug  = self::$settings_page;
			$func  = __CLASS__ . '::render';

			add_submenu_page( 'options-general.php', $title, $title, $cap, $slug, $func );
		}

        if ( current_user_can( 'manage_network_plugins' ) ) {

            $title = __('Divi Monk', 'divi-monk');
    		$cap   = 'manage_network_plugins';
    		$slug  = self::$settings_page;
    		$func  = __CLASS__ . '::render';

    		add_submenu_page( 'settings.php', $title, $title, $cap, $slug, $func );
        }
	}

    /**
	 * Renders the admin settings.
	 *
	 * @since 1.1.5
	 * @return void
	 */
	static public function render()
	{
		include DIVI_MONK_DIR . 'includes/admin-settings.php';
	}

    /**
	 * Renders the update message.
	 *
	 * @since 1.1.5
	 * @return void
	 */
	static public function render_update_message()
	{
		if ( ! empty( self::$errors ) ) {
			foreach ( self::$errors as $message ) {
				echo '<div class="error divi-monk-message"><p>' . $message . '</p></div>';
			}
		}
		else if ( ! empty( $_POST ) && ! isset( $_POST['email'] ) && ! isset( $_POST['divi_monk_license_key'] ) ) {
			echo '<div class="updated divi-monk-message"><p>' . esc_html__( 'Settings updated!', 'divi-monk' ) . '</p></div>';
		}
	}

    /**
	 * Renders the action for a form.
	 *
	 * @since 1.1.5
	 * @param string $type The type of form being rendered.
	 * @return void
	 */
	static public function get_form_action( $type = '' )
	{
		if ( is_network_admin() ) {
			return network_admin_url( '/settings.php?page=' . self::$settings_page . $type );
		}
		else {
			return admin_url( '/options-general.php?page=' . self::$settings_page . $type );
		}
	}

    /**
	 * Adds an error message to be rendered.
	 *
	 * @since 1.1.5
	 * @param string $message The error message to add.
	 * @return void
	 */
	static public function add_error( $message )
	{
		self::$errors[] = $message;
	}

    /**
	 * Returns an option from the database for
	 * the admin settings page.
	 *
	 * @since 1.0.0
	 * @param string $key The option key.
     * @param bool $network_override Multisite template override check.
	 * @return mixed
	 */
    static public function get_option( $key, $network_override = true )
    {
    	if ( is_network_admin() ) {
    		$value = get_site_option( $key );
    	}
        elseif ( ! $network_override && is_multisite() ) {
            $value = get_site_option( $key );
        }
        elseif ( $network_override && is_multisite() ) {
            $value = get_option( $key );
            $value = false === $value ? get_site_option( $key ) : $value;
        }
        else {
    		$value = get_option( $key );
    	}

        return $value;
    }

    /**
	 * Updates an option from the admin settings page.
	 *
	 * @since 1.0.0
	 * @param string $key The option key.
	 * @param mixed $value The value to update.
	 * @param bool $network_override Multisite template override check.
	 * @return mixed
	 */
    static public function update_option( $key, $value, $network_override = true )
    {
    	if ( is_network_admin() ) {
    		update_site_option( $key, $value );
    	}
		else if ( $network_override && is_multisite() ) {
			delete_option( $key );
		}
        else {
    		update_option( $key, $value );
    	}
    }

    /**
	 * Delete an option from the admin settings page.
	 *
	 * @since 1.1.5
	 * @param string $key The option key.
	 * @param mixed $value The value to delete.
	 * @return mixed
	 */
    static public function delete_option( $key )
    {
    	if ( is_network_admin() ) {
    		delete_site_option( $key );
    	} else {
    		delete_option( $key );
    	}
    }

    static public function is_valid_page()
    {
        if ( is_admin() && isset( $_GET['page'] ) && self::$settings_page == $_GET['page'] ) {
            return true;
        }

        return false;
    }

    /**
	 * Saves the admin settings.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function save()
	{
		// Only admins can save settings.
		if ( ! current_user_can('manage_options') ) {
			return;
		}

        self::save_license();
	}

    /**
	 * Saves the license.
	 *
	 * @since 1.0
	 * @access private
	 * @return void
	 */
    static private function save_license()
    {
        if ( ! isset( $_POST['divi_monk_license_nonce'] ) || ! wp_verify_nonce( $_POST['divi_monk_license_nonce'], '_divi_monk_license_nonce' ) ) {
			return;
        }

        if ( isset( $_POST['divi_monk_license_key'] ) ) {

        	$old = divi_monk_get( '_divi_monk_license_key' );
            $new = $_POST['divi_monk_license_key'];

        	if ( $old && $old != $new ) {
        		delete_option( '_divi_monk_license_status' ); // new license has been entered, so must reactivate
        	}

            divi_monk_update( '_divi_monk_license_key', $new );
        }
    }

    static public function get_templates( $type = 'layout' )
    {
        if ( in_array( $type, self::$templates_type ) ) {
            return get_option( '_divi_monk_templates_' . $type, false );
        }
    }

    /**
	 * Returns an array of all templates that are enabled.
	 *
	 * @since 1.1.5
	 * @return array
	 */
    static public function get_enabled_templates()
    {
        $enabled_templates = get_option( '_divi_monk_enabled_templates', false );

        if ( is_array( $enabled_templates ) && count( $enabled_templates ) ) {

            return $enabled_templates;
        }

        return array();
    }

    static public function download_templates_data( $template_type = 'layout' )
    {
        if ( ! self::is_valid_page() ) {
            return;
        }

        $remote_url = DIVI_MONK_REMOTE;
        $remote_url .= $template_type . '?v=' . rand();

        // Get template data response.
        $response = wp_remote_get( $remote_url, array(
            'timeout'       => 30,
            'sslverify'     => false,
            'httpversion'   => '1.1',
        ) );

        if ( is_wp_error( $response ) ) {
            self::add_error( $response->get_error_message() );
            return 'wp_error';
        }

        // Retrieve template data.
        $data = json_decode( wp_remote_retrieve_body( $response ), 1 );

        // Return if no valid data.
        if ( array_key_exists( 'code', $data ) ||
                array_key_exists( 'message', $data ) ||
                array_key_exists( 'data', $data ) ||
                ! count( $data )
            )
        {
            self::add_error( sprintf( esc_html__('%s templates are not available at the moment.', 'divi-monk'), ucfirst( $template_type ) ) );
            return 'failed';
        }

        if ( count( $data ) ) {
            update_option( '_divi_monk_templates_' . $template_type, $data, true );
            return 'success';
        }
    }

    static public function check_templates_data()
    {
        if ( ! self::is_valid_page() ) {
            return;
        }

        if ( isset( $_GET['refresh'] ) ) {
            foreach ( self::$templates_type as $type ) {
                self::download_templates_data( $type );
            }
            return;
        }

        $template_type = 'layout';

        // Get template type.
        if ( isset( $_GET['navigate'] ) && in_array( $_GET['navigate'], self::$templates_type ) ) {
            $template_type = $_GET['navigate'];
        }

        $templates_data = get_option( '_divi_monk_templates_' . $template_type );

        if ( false === $templates_data || ! is_array( $templates_data ) ) {
            return self::download_templates_data( $template_type );
        }
    }

    /**
     * Delete the options of enabled page templates.
     *
     * @since 1.1.6
     * @return void
     */
    static public function flush_templates()
    {
        if ( isset( $_GET['page'] ) && self::$settings_page == $_GET['page'] ) {

            // Enabled or installed templates.
            if ( isset( $_GET['flush_enabled_templates'] ) ) {
                delete_option( '_divi_monk_enabled_templates' );
            }

            // All downloaded templates.
            if ( isset( $_GET['flush_templates'] ) && 'all' == $_GET['flush_templates'] ) {
                foreach ( self::$templates_type as $type ) {
                    delete_option( '_divi_monk_templates_' . $type );
                }
            }

            // Templates by type.
            if ( isset( $_GET['flush_templates'] ) && '' != $_GET['flush_templates'] ) {
                delete_option( '_divi_monk_templates_' . $_GET['flush_templates'] );
            }
        }
    }

    static public function flush_license()
    {
        if ( isset( $_GET['page'] ) && self::$settings_page == $_GET['page'] && isset( $_GET['flush_license'] ) ) {
            divi_monk_delete( '_divi_monk_license_key' );
            divi_monk_delete( '_divi_monk_license_status' );
        }
    }

}

Divi_Monk_Admin_Settings::init();
