<?php
/**
 * Handles the logic for Ajax.
 *
 * @since 1.0.0
 */
class Divi_Monk_Ajax {
    /**
     * Holds error messages.
     *
     * @since 1.0.0
     */
    static public $errors = array();

    /**
     * Initialize hooks and filters.
     *
     * @since 1.0.0
     */
    static public function init()
    {
        add_action( 'wp_ajax_dm_import_template', __CLASS__ . '::import_template' );
    }

    /**
     * Get template data and import.
     *
     * @since 1.0.0
     */
    static public function import_template()
    {
        // Set error if template id is empty or invalid.
        if ( ! isset( $_POST['template_id'] ) || ! absint( $_POST['template_id'] ) ) {
            self::$errors[] = esc_html__( 'Invalid template ID.', 'divi-monk' );
        }

        // Set error if nonce field is empty or invalid.
        if ( ! isset( $_POST['template_nonce'] ) || ! wp_verify_nonce( $_POST['template_nonce'], 'dm_template_' . absint( $_POST['template_id'] ) ) ) {
            self::$errors[] = esc_html__( 'Invalid template data supplied.', 'divi-monk' );
        }

        $body = array();

        // Check if request have page and timestamp args
        // and build an array to pass these to get template data.
        if ( isset( $_POST['page'] ) && isset( $_POST['timestamp'] ) ) {
            $body = array(
                'page'      => $_POST['page'],
                'timestamp' => $_POST['timestamp'],
                'file'      => null
            );
        }

        // Get the template data.
        $template_data = self::fetch_template( absint( $_POST['template_id'] ), $body );

        // If failure to fetch template, set an error.
        if ( ! $template_data ) {
            self::$errors[] = esc_html__( 'Template is not available at the moment.', 'divi-monk' );
        }

        // Return if there are errors.
        if ( count( self::$errors ) ) {
            wp_send_json_error( array( 'errors' => self::$errors ) );
        }

        // Check for Paginate images.
        if ( is_array( $template_data ) && isset( $template_data['page'] ) ) {
            wp_send_json( $template_data );
        }

        // Import template.
        $portability = new Divi_Monk_Portability( sanitize_text_field( $_POST['context'] ) );
        $portability->import( $template_data );
        die;
    }

    /**
     * Fetch template data.
     *
     * @since 1.0.0
     *
     * @param int $template_id
     * @param array $body
     *
     * @return mixed
     */
    static private function fetch_template( $template_id = 0, $body = array() )
    {
        // Sanitize template id.
        $template_id = absint( $template_id );

        if ( ! $template_id ) {
            $template_id = absint( $_POST['template_id'] );
        }

        // Set remote URL.
        $remote_url = DIVI_MONK_REMOTE . 'template/';
        $remote_url .= $template_id . '?v=' . rand();

        $data = '';

        // Arguments to pass in wp_remote_post.
        $args = array(
            'method'        => 'POST',
            'timeout'       => 120,
            'sslverify'     => false,
            'httpversion'   => '1.1',
        );

        // If body param exists, merge it with arguments.
        if ( is_array( $body ) && count( $body ) ) {
            $args['body'] = $body;
        }

        // Get template data response.
        $response = wp_remote_post( $remote_url, $args );

        if ( is_wp_error( $response ) ) {
            self::$errors[] = $response->get_error_message();
        }
        else {
            // Retrieve template data.
            $data = json_decode( wp_remote_retrieve_body( $response ), 1 );
        }

        // No valid data.
        if ( is_array( $data ) && count( $data ) ) {

            if ( array_key_exists( 'code', $data ) ||
                    array_key_exists( 'message', $data ) ||
                    array_key_exists( 'data', $data )
                )
            {
                self::$errors[] = esc_html__('Template is not available at the moment.', 'divi-monk');
            }
        }

        // Return false if errors.
        if ( count( self::$errors ) ) {
            return 0;
        }

        // Return data.
        return $data;
    }
}

// Initialize Divi_Monk_Ajax class.
Divi_Monk_Ajax::init();
